package com.fjhx.tenant.controller.tenant;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.tenant.entity.tenant.dto.BindingMenuDto;
import com.fjhx.tenant.entity.tenant.dto.TenantInfoDto;
import com.fjhx.tenant.entity.tenant.dto.TenantInfoSelectDto;
import com.fjhx.tenant.entity.tenant.po.TenantInfo;
import com.fjhx.tenant.entity.tenant.vo.TenantInfoVo;
import com.fjhx.tenant.service.tenant.TenantInfoService;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.DatasourceConstant;
import com.ruoyi.common.core.domain.BaseSelectDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * <p>
 * 租户表 前端控制器
 * </p>
 *
 * @author zlj
 * @since 2023-03-15
 */
@TenantIgnore
@DS(DatasourceConstant.SLAVE_NAME)
@RestController
@RequestMapping("/tenantInfo")
public class TenantInfoController {

    @Autowired
    private TenantInfoService tenantInfoService;

    /**
     * 租户表分页
     */
    @PostMapping("/list")
    public List<TenantInfo> list(@RequestBody TenantInfoSelectDto dto) {
        return tenantInfoService.getList(dto);
    }

    /**
     * 租户表分页
     */
    @PostMapping("/page")
    public Page<TenantInfoVo> page(@RequestBody TenantInfoSelectDto dto) {
        return tenantInfoService.getPage(dto);
    }

    /**
     * 租户表明细
     */
    @PostMapping("/detail")
    public TenantInfoVo detail(@RequestBody BaseSelectDto dto) {
        return tenantInfoService.detail(dto.getId());
    }

    /**
     * 租户表新增
     */
    @PostMapping("/add")
    public void add(@RequestBody TenantInfoDto tenantInfoDto) {
        tenantInfoService.add(tenantInfoDto);
    }

    /**
     * 租户表编辑
     */
    @PostMapping("/edit")
    public void edit(@RequestBody TenantInfoDto tenantInfoDto) {
        tenantInfoService.edit(tenantInfoDto);
    }

    /**
     * 租户表删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        tenantInfoService.delete(dto.getId());
    }

    /**
     * 租户绑定菜单
     */
    @PostMapping("/bindingMenu")
    public void bindingMenu(@RequestBody @Validated BindingMenuDto dto) {
        tenantInfoService.bindingMenu(dto);
    }

}
